/*	
 *	addShop() :: setting up the shopping cart with Paypal
 *
 *	Usage: $(element).addShop( [items,] options );
 *
 *	Dual licensed under the MIT and GPL licenses.
 *	- http://www.opensource.org/licenses/mit-license.php
 *	- http://www.gnu.org/copyleft/gpl.html
 *
 *	Items: jQuery element (array) holding the elements
 *
 *	Options:
		target: 'ShoppingCart',
		currency: 'EUR',
		gateway: 'paypal',
		locale: 'US',
		quantityCap: 0,
		shippingFlat: false
 */

(function($) {
	'use strict';
		
	$.fn.addShop = function(newItems, settings) {

		if (!$.isArray(newItems)) {
			settings = newItems;
			newItems = [];
		}
			
		settings = $.extend( {}, $.fn.addShop.defaults, settings );
		
		if (!settings.id) {
			return this;
		}
			
		var id = $.fn.addShop.id,
			text = $.fn.addShop.text,
			seller = settings.id.replace('|','@'),
			url = (DEBUG? $.fn.addShop.st.sandboxurl : $.fn.addShop.st.url),
			currency = $.fn.addShop.st.curr_symbol[settings.currency] || settings.currency,
			t = $(this).eq(0), 	// target
			ns, 				// namespace
			f, fs,	 			// forms 
			sd, 				// saved discount
			el, 				// generic jQuery el
			target = $.fn.addShop.st.targetCart,
			itemCont,			// item container
			items = $(),		// items
			options,			// shop options array
			
			// Cleaning up old event handlers (this supposed to never happen)
			
			cleanup = function(ns) {
				if (ns) {
					$(window).add(t.find('form')).add(t.find('a')).add(t.find('button')).off('.' + ns);
				}
			},
			
			// Reading shop options
			
			getOptions = function(s) {
				var v = s.split('::'), 
					i, k, p, o, op = [];
			
				for (i = 0; i < v.length; i++) {
					k = v[i].split('=');
					if ( k.length > 1 ) {
						o = {};
						o.label = k[0];
						p = k[1].split('+');
						// Price
						if ((o.price = parseFloat(p[0])) == null)
							continue;
						// Shipping
						if (p.length > 1) {
							o.shipping = parseFloat(p[1]);
						} else {
							o.shipping = 0;
						}
						// Shipping 1+
						if (p.length > 2) {
							o.shipping2 = parseFloat(p[2]);
						} else {
							o.shipping2 = o.shipping;
						}
						// Display text
						o.text = k[0] + ' = ' + currency + ' ' + o.price.toFixed(2) + ((p.length > 1)? ('+' + o.shipping.toFixed(2)) : '');   
						op.push(o);
					}
				}
				
				return op;
			},
		
			// Adding select box
			
			addOptions = function(f, op) {
				var i, e = $('<select>').appendTo(f);
				
				for (i = 0; i < op.length; i++) {
					e.append($('<option>', {
						val: op[i].price,
						html: op[i].text
					}));
				}
				
				return e;
			},
		
			// Adding an input
			
			addInput = function(f, name, val, type, attr) {
				
				if (!f || !name) {
					return {};
				}
				
				var e = $('<input>', { 
						type: type || 'text'
					}).appendTo(f);
							
				// name
				e.prop('name', name); 
					
				// initial value
				if (val) {
					e.val((typeof val === 'string')? val.stripQuote() : val);
				}
					
				// simple attributes e.g. 'readonly'
				if (attr) {
					e.prop(attr, true);
				}
	
				return e;
			},

			// Reading or setting discount rate
			
			discountRate = function(v) {
				if (typeof v === UNDEF) {
					return value(id.discountRateCart);
				} else {
					value(id.discountRateCart, v);
				}
			},
		
			// Reading or setting discount amount
			
			discountAmount = function(v) {
				if (typeof v === UNDEF) {
					return value(id.discountAmountCart);
				} else {
					value(id.discountAmountCart, v);
				}
			},
		
			// Adding discount coupon name to the sent name
			
			discountName = function(name, val) {
				var el = fs.children('[name=' + id.option + ']');
				if (el && el.length) {
					var s = el.val().replace(/(\s\(CC\:.+\))$/, '');
					el.val(s + ' (CC:' + name + ((typeof val !== UNDEF)? ('=' + val) : '') + ')');
				}
			},
			
			// set / get float value from fs
			
			value = function(id, v) {
				var el = fs.children('[name=' + id + ']');
				if (typeof v === UNDEF) {
					// get
					return (el && el.length)? parseFloat(el.val()) : null;
				} else if (v == null) {
					// remove
					el.remove();
				} else {
					// set
					if (el && el.length) {
						el.val(v);
					} else {
						addInput(fs, id, v, 'hidden');
					}
				}
			},
		
			// currently selected option's index
			
			curr = function() {
				var el = f.children('select').eq(0);
				return (el && el.length)? el.prop('selectedIndex') : 0;
			},
		
			// Shows total (discounted) price
			
			showTotal = function() {
				var i = curr(),
					ed = f.children('.discount').eq(0),
					et = f.children('.total').eq(0),
					q = f.children('[name=copies]').val() || 1,
					pr = options[i].price,
					sh = options[i].shipping + (q - 1) * options[i].shipping2,
					cnt = items.length,
					da = discountAmount(),
					dr = discountRate();
				
				if (!ed.length) {
					ed = $('<span>', {
						'class': 'discount'
					}).insertAfter(f.children('select'));
				}
						
				if (da && da > 0) {
					ed.show().html('- ' + currency + ' ' + da.toFixed(2));
					et.html(currency + ' <b>' + (cnt * (q * pr + sh) - da).toFixed(2) + '</b>');
				} else if (dr && dr > 0) {
					ed.show().text('-' + dr + '%');
					et.html(currency + ' <b>' + (cnt * (q * pr * (100 - dr) / 100 + sh)).toFixed(2) + '</b>');
				} else {
					ed.hide();
					et.html(currency + ' <b>' + (cnt * (q * pr + sh)).toFixed(2) + '</b>');
				}
				
			},
			
			// Adjusting shipping fee
			/*
			adjustShipping = function(v) {
				if (typeof v === UNDEF) {
					v = options[curr()].shipping;
				}
				if (cnt > 1) {
					value(id.handlingCart, (v || 0) * cnt + (settings.handling || 0));
				} else {
					value(id.shipping, v);
					if (!settings.shippingFlat) {
						value(id.shipping2, v);
					}
				}					
			},
			*/
		
			// Select box or Copies has Changed
			
			change = function() {
				var i = curr(),
					q = f.children('[name=copies]').val() || 1,
					pr = options[i].price,
					sh = options[i].shipping,
					sh2 = options[i].shipping2;
					
				if ( settings.quantityCap && q > settings.quantityCap ) {
					f.children('[name=copies]').val(q = settings.quantityCap);
				}
				
				value(id.name, items.length + ' ' + text.items);
				
				$('[name^=' + id.name + '_]').each(function(i) {
					var j = i + 1;
					value(id.price + '_' + j, pr);
					value(id.copies + '_' + j, q);
					value(id.shipping + '_' + j, sh? sh : null);
					value(id.shipping2 + '_' + j, sh2? sh2 : null);
				});
				
				value(id.option, options[i].text);
				
				showTotal();
			},
		
			// Validate coupon code
			
			validateCoupon = function(feedback) {
				var el, now = new Date(), v, fb = feedback === true;
				
				if (settings.coupons && (el = f.children('[name=coupon]')) && (v = el.val().trim()).length) {
					var i, c, d, 
						cs = xDecrypt(settings.coupons).split('::');
						
					for (i = 0; i < cs.length; i++) {
						
						c = cs[i].split(/=|\s*<\s*/);
						
						if (c[0] === v && c.length > 1) {
							
							d = parseFloat(c[1]);
							if ( d < 0.01 ) {
								// Too small
								continue;
							}
							
							if (c.length > 2) {
								// Checking expiry
								var dt, ex = c[2].split(/-|:|\//);
								if (ex.length < 2) {
									ex[1] = 1;
								}
								if (ex.length < 3) {
									ex[2] = 1;
								}
								dt = new Date(parseInt(ex[0]),parseInt(ex[1]),parseInt(ex[2]));
								
								if (dt < now) {
									// Expired
									$('body').addModal($('<div>', {
										html: text.expired.replace('{0}', v)
									}), {
										type: 'error'
									});
									
									return false;
								}
							}
														
							if (c[1].charAt(c[1].length - 1) === '%') {
								
								// Discount rate
								
								if (d > 99 || d < 1) {
									// Not allowed
									continue;
								}
								
								var dr = discountRate();
								
								if (dr > d) {
									// Lower than current discount
									if (fb) {
										$('body').addModal($('<div>', {
											html: text.lowerThanCurrent.replace('{0}', dr + '%')
										}), {
											type: 'warning'	
										});
									}
								} else {
									// Higher
									if (fb) {
										// Redeem feedback
										$('body').addModal($('<div>', {
											html: text.accepted.replace('{0}', d + '%')
										}), {
											title: text.success	
										});
									}
									
									discountRate(d);
									discountName(v, d + '%');
									$.cookie('discountRate', v, 86400);
								}
	
							} else if ($.cookie('discount_' + v)) {
								
								// The discount amount has been reclaimed already
								
								if (fb) {
									// Redeem: warning, but no action
									$('body').addModal($('<div>', {
										html: text.reclaimed
									}), {
										type: 'warning'
									});
								} 
								
								// Removing discount amount, adding back the saved rate
								discountAmount(null);
								if (sd) {
									discountRate(sd);
									discountName(v, sd + '%');
								}
								
							} else {
								
								// Discount amount (coupon is not yet used in the past 24 hours)
								
								var dr = discountRate(),
									pr = options[curr()].price,
									cnt = items.length,
									da = discountAmount() || ((dr > 0)? (cnt * pr * dr / 100) : 0);
								
								if ((pr * cnt) < d) {
									// Price is lower than discount amount : no go
									if (fb) {
										// Redeem: warning, but no action
										$('body').addModal($('<div>', {
											html: text.higherThanPrice.replace('{0}', currency + '&nbsp;' + d)
										}), {
											type: 'warning'
										});
									}
									
								} else if (da > d) {
									// Previous discount amount is lower than current
									if (fb) {
										// Redeem: warning, but no action
										$('body').addModal($('<div>', {
											html: text.lowerThanCurrent.replace('{0}', currency + '&nbsp;' + da.toFixed(2))
										}), {
											type: 'warning'	
										});
									}
									
								} else {
								
									if (fb) {
										// Redeem: feddback
										$('body').addModal($('<div>', {
											html: text.accepted.replace('{0}', currency + '&nbsp;' + d)
										}), {
											title: text.success
										});
									} else {
										// Saving the coupon for later check
										$.cookie('discount_' + v, d, 86400);
										// Saving current discount rate
										sd = discountRate();
										discountRate(null);
									}
									
									// Adding discount amount
									discountAmount(d);
									discountName(v, settings.currency + ' ' + d);
									
								}
							}
							
							showTotal();
							return true;
						}
					}
					
					// Not found
					$('body').addModal($('<div>', {
						html: text.noSuch
					}), {
						type: 'error'			
					});
					// Do not send the form!
					return false;
				}
				
				// No coupons exists or coupon field is empty
				return true;
			},
		
			
			// Adding new items
			
			addItems = function(newItems) {
				
				if (!newItems || !newItems.length) {
					return;
				}
				
				var max = items.length;
				
				for (var s, i = 0, l = newItems.length; i < l; i++) {
					s = Album.getThumbPath(newItems[i]);
					itemCont.append($('<a>').append($('<img>', {
						src: s
					})));
					value(id.name + '_' + (max + i + 1), Album.getItemPath(newItems[i])); 
				};
				
				items = itemCont.children();
				max = items.length;
				t.toggle(max > 0);
				$(settings.buttons.emptyCart).toggleClass('disabled', !max);
				f.find('.count b').text(max);
				change();
				
			},
			
			// Emptying cart
			
			emptyCart = function() {
				
				itemCont.empty();
				items = $();
				fs.children('[name^="' + id.name + '_"],[name^="' + id.price + '_"],[name^="' + id.copies + '_"]').remove();
				t.hide();
				$(settings.buttons.emptyCart).addClass('disabled');
				f.find('.count b').text('0');
				change();
			},
			
			// Removing one item
			
			removeItem = function(item) {
				
				var max = items.length,
					n = items.index(item) + 1,
					el = fs.children('[name^="' + id.name + '_"],[name^="' + id.price + '_"],[name^="' + id.copies + '_"]');
				
				// removing the current element
				item.remove();
				el.filter('[name$="_' + n + '"]').remove();
				
				if (n < max) {
					
					// renaming all below
					for (var i = n + 1; i <= max; i++) {
						el.filter('[name="' + id.name + '_' + i + '"]').attr('name', id.name + '_' + (i - 1));
						el.filter('[name="' + id.price + '_' + i + '"]').attr('name', id.price + '_' + (i - 1));
						el.filter('[name="' + id.copies + '_' + i + '"]').attr('name', id.copies + '_' + (i - 1));
					}
				}
				
				items = itemCont.children();
				max = items.length;
				t.toggle(max > 0);
				$(settings.buttons.emptyCart).toggleClass('disabled', !max);
				f.find('.count b').text(max);
				change();
			};
		
		/* ***************************************
		 *          Creating the elements 
		 * *************************************** */
		 
		// Shop options
		 
		options = getOptions(settings.options);
		
		if (!options.length) {
			return;
		}
		
		// Initializing the items container
		
		t.hide();
		
		// Creating new namespace
		
		cleanup(t.data('las_ns'));
		$(this).data('las_ns', ns = 'las_' + Math.floor(Math.random() * 10000));		
		
		// New container
		
		itemCont = $('<div>', {
			'class': settings.shopItemsHook || $.fn.addShop.st.shopItemsHook
		}).appendTo(t);
		
		addItems(newItems);
		
		itemCont.on('click', '> a', function() {
			removeItem($(this));
			return false;
		});
		
		// Adding empty cart button
		/*
		el = $('<a>', {
			'class': 'icon-trash',
			text: text.emptyCart
		}).on('click.' + ns, emptyCart);
		
		t.append( $('<div>', {
			'class': 'buttons'
		}).append( $('<div>', {
			'class': 'options'
		}).append(el) ) );
		*/

		// Wiring empty cart button
		
		$(settings.buttons.emptyCart).on('click.' + ns, emptyCart);
		
		/*
		 *	Creating the form to display
		 */
		
		f = $('<form>', {
			name: 'shopping',
			method: 'post'
		}).appendTo(t);
		
		// Count
		f.append($('<span>', {
			'class': 'count',
			html: '<b>' + items.length + '</b> &times;' 
		}));
		
		// Options
		el = addOptions(f, options);
		el.on('change', change);

		// Discount
		if (!settings.hasOwnProperty('discount')) {
			settings.discount = $.fn.addShop.defaults.discount || 0;
		}
		if (settings.discount === '-') {
			settings.discount = 0;
		}
		/*	
		if (settings.discount && settings.discount < 100) {
			f.append($('<span>', {
				'class': 'discount',
				text: '-' + settings.discount + '%'
			}));
		}
		*/
		// Quantity
		if (settings.quantityCap !== 1) {
			f.append('&times;');
			el = addInput(f, 'copies', 1);
			el.addClass('copies').on('change', change);
		}
		
		// Total
		f.append('=');
		f.append($('<span>', {
			'class': 'total'
		}));
		
		// Coupon
		if (settings.coupons) {
			f.append($('<input>', {
				type: 'text',
				name: 'coupon',
				'class': 'coupon',
				placeholder: text.couponCode
			}));
			// Redeem btn
			var b = $('<a>', {
				href: NOLINK,
				html: '&nbsp;',
				'class': 'redeem icon-refresh'
			}).on('click.' + ns, function(e) {
				e.preventDefault();
				validateCoupon(true);
				return false;
			});
			/*
			if (!document.touchMode) {
				b.addTooltip(text.redeem);
			}
			*/
			f.append(b);
		}
					
		/*
		 *	Creating the form to be sent to Paypal
		 */
			
		fs = $('<form>', {
			name: id.form,
			action: url + 'cgi-bin/webscr/',
			method: 'post'
		}).appendTo(t);
		
		if (!settings.sameWindowCheckout) {
			fs.attr('target', target);
		}
			
		value('cmd', '_cart');
		value('charset', 'utf-8');
		value('lc', settings.locale);
		value(id.seller, seller);
		value(id.currency, settings.currency);
		value(id.shopUrl, settings.continueUrl || decodeURIComponent(window.location.href));
		if (settings.handling != null && $.isNumeric(settings.handling)) {
			value(id.handlingCart, settings.handling);
		}
		value(id.option, options[0].label);
		
		if (settings.discount && settings.discount < 100) {
			discountRate(settings.discount);
		}
		
		value('upload', 1);
		//value(id.priceCart, options[0].price * items.length);
		//value(id.handlingCart, (options[0].shipping || 0) * cnt + (settings.handling || 0));
				
		change();
		
		fs.append($('<button>', {
			id: 'shopAdd',
			name: 'submit',
			'class': 'paypalbtn button icon-paypal',
			html: text.buyNow
		}));
		
		// Submit event handling
		
		fs.on('submit.' + ns, function(e) {
			var succ = true, 
				el = $(e.target);
			
			// Checking invalid coupons
			if (settings.coupons) {
				succ = validateCoupon();
			}
			if (succ) {
				// Yes, ready to send
				el.parents('[role=modal]').trigger('destroy');
				if (settings.sameWindowCheckout) {
					fs.off('submit.' + ns);
					fs.trigger('submit');
				} else {
					//if (!settings.continueUrl) {
					window.open('', target, 'width=1024,height=600,scrollbars=yes,resizable=yes,menubar=no,toolbar=no,directories=no,status=no,copyhistory=no');
					return true;
				}
			}
			return succ;
		});
			
		// Checking if there's a saved coupon
		
		if (settings.coupons) {
			var v = $.cookie('discountRate');
			if (v) {
				$('input[name=coupon]').val(v);
				validateCoupon(false);
			}
		}
						
		// Initializing total amount
		
		showTotal();
		
		// Creating a new 'addItems' action
		
		t.on('addItems.' + ns, function(e) {
			addItems(Array.prototype.slice.call(arguments, 1));
			return false;
		});

		return this;
	};
	
	$.fn.addShop.defaults = {
		currency: 'EUR',
		locale: 'US',
		quantityCap: 0,
		shippingFlat: false,
		sameWindowChekout: false,
		shopItemsHook: 'cart-items'
	};
		
	$.fn.addShop.text = getTranslations({
		addCart: 'Add to Cart',
		viewCart: 'View Cart',
		emptyCart: 'Empty Cart',
		buyNow: 'Buy Now',
		items: 'items',
		success: 'Success',
		couponCode: 'Coupon code',
		redeem: 'Redeem',
		noSuch: 'No such coupon exists!',
		expired: 'The coupon code <b>{0}</b> has expired!',
		accepted: 'The coupon code is accepted. You will get <b>{0}</b> discount the next time you add this item to the cart.',
		lowerThanCurrent: 'This coupon offers lower discount than the current <b>{0}</b>.',
		higherThanPrice: 'You can use this coupon only for items priced higher than <b>{0}</b>.',
		reclaimed: 'This coupon has already been used!'
	});
	
	// Static vars
	$.fn.addShop.st = {
		target: 	'ShoppingCart',
		targetCart:	'BuyNow',
		url: 		'https://www.paypal.com/',
		sandboxurl:	'https://www.sandbox.paypal.com/',
		curr_symbol: {
			'USD': 'US$',
			'EUR': '&euro;',
			'GBP': 'GB&pound;',
			'JPY': '&yen;',
			'HUF': 'Ft'
		}
	};
	
	// Mapping names
	$.fn.addShop.id = {
		'form':				'paypal',
		'seller':			'business',
		'currency':			'currency_code',
		'name':				'item_name',
		'option':			'item_number',
		'custom':			'custom',
		'price':			'amount',
		'priceCart':		'amount_1',
		'copies':			'quantity',
		'discountRate':		'discount_rate',
		'discountRateCart':	'discount_rate_cart',
		'discountAmount':	'discount_amount',
		'discountAmountCart':'discount_amount_cart',
		'shipping':			'shipping',
		'shipping2':		'shipping2',
		'handlingCart':		'handling_cart',
		'shopUrl':			'shopping_url'
	};			

})(jQuery);

/*
 * Simple decryption
 */

var xDecrypt = function(c) {
	
	if (!typeof this === 'String') {
		return '';
	}
	var xs = [0x93,0xA3,0x57,0xFE,0x99,0x04,0xC6,0x17];
	var cl = c.length,
		sl = Math.ceil(cl / 8) * 5;
	var src = new Array(sl), r = '', i, j = 0, k, v;
	for (i = 0; i < sl; i++) {
		src[i] = 0;
	}
	for (i = 0; i < cl; i++) {
		if ((v = c.charCodeAt(i) - 0x30) > 9) {
			v -= 7;
		}
		v <<= 11 - j % 8;
		k = Math.floor(j / 8);
		if (k < sl) {
			src[k] |= v >> 8;
			if (++k < sl)
				src[k] |= v & 0xff;
		}
		j += 5;
	}
	for (i = 0; i < sl; i++) {
		src[i] ^= xs[i % 8];
	}
	sl = src[0] | (src[1] << 8);
	for (v = 0, i = 4; i < sl; i++) {
		r += String.fromCharCode(src[i]);
		v += src[i];
	}
	if (v != ((src[2] & 0xff) | (src[3] << 8)))
		r = '';
	return r;
};
