/*	
 *	addTooltip() :: little Popup displaying 'title' text, or passed text (can be HTML)
 *
 *	Copyright by Lazaworx
 *	http://www.lazaworx.com
 *	Author: Laszlo Molnar
 *
 *	Dual licensed under the MIT and GPL licenses.
 *	- http://www.opensource.org/licenses/mit-license.php
 *	- http://www.gnu.org/copyleft/gpl.html
 *
 *	Usage: $(element).addTooltip( [txt,] options );
 *	options:
		id: 'hint',
		stay: 3000,
		posX: ALIGN_CENTER,
		posY: ALIGN_BOTTOM,
		toX: ALIGN_CENTER,
		toY: ALIGN_TOP
 */

(function($) {
	'use strict';
			
	$.fn.addTooltip = function(content, settings) {
		
		if (content && typeof content !== 'string' && !content.jquery) {
			settings = content;
			content = null;
		}
		
		var id = {
				tooltip: 'tooltip'
			},
			ns = '.tt';

		settings = $.extend( {}, $.fn.addTooltip.defaults, settings );
		
		$.fn.hideAllTooltips = function() {
			$('.' + settings.className).hide();
		};
		
		var createNew = function(el, cont) {
			var c;
			
			if (!cont) {
				
				if (cont = el.data(id.tooltip)) {
					// read from related layer
					if (cont.charAt(0) === '.') {
						cont = el.find(cont);
					} else if (cont.charAt(0) === '#') {
						cont = $(cont);
					}
					if (cont.jquery) {
						cont.removeClass('hidden');
					}
				} else {
					// read from data or title attr
					cont = el.attr('title');
					el.removeAttr('title');
				}
				
				if (!cont || !cont.length) {
					return null;
				}
				
				c = $('<div>', {
					html: cont
				}).appendTo('body');
				
			} else if (typeof cont === 'string') {
				
				// passed directly :: html structure as string
				c = $('<div>', {
					html: cont
				}).appendTo('body');
				
			} else if (cont.jquery) {
				
				// jQuery element
				if (!$.contains(document.body, cont[0])) {
					c = cont.appendTo('body');
				} else {
					c = cont;
				}
				
			} else {
				return null;
			}
			
			c.addClass(settings.className).hide();
			c.append($('<span>', {
				'class': settings.nub
			}));
			return c;
			
		};
		
		return this.each(function() {
				
			var self = $(this), 
				cont,
				to, rto,			// timeout, remove timeout
				over = false,		// over the tooltip 
				focus = false,		// tooltip input got focus
				offs;				// last offset
						
			// getFocus
			var getFocus =  function() {
				rto = clearTimeout(rto);
				over = true;
				cont.finish().show();
			};
			
			// lostFocus
			var lostFocus = function() {
				if ( focus ) {
					return;
				}
				clearTimeout(rto);
				over = false;
				rto = setTimeout(remove, 100);
			};
				
			// Leaving the trigger element
			var leave = function(e) {
				to = clearTimeout(to);
				clearTimeout(rto);
				over = false;
				rto = setTimeout(remove, 100);
			};
						
			// The hotspot clicked
			var clicked = function(e) {
				leave();
				return true;
			};
			
			// Fading the popup
			var remove = function() {
				if ( !over && cont ) {
					cont.stop(true, false).fadeOut(200, function() { 
						cont.hide();
					});
				}
			};
			
			// Remove later :: automatically on touch devices
			var removeLater = function() {
				clearTimeout(rto);
				rto = setTimeout(remove, settings.stay);
			}
			
			// Create
			var create = function() {
				
				if (!(cont = createNew(self, content))) {
					return false;
				}
				
				// Keep the popup live while the mouse is over
				if (!hasTouch) {
					cont.on('mouseover' + ns, getFocus)
						.on('mouseout' + ns, lostFocus);
				}
				// ... or an input box has focus
				cont.find('input, textarea').on('focus' + ns, function() {
					focus = true;
					getFocus();
				}).on('blur' + ns, function() {
					focus = false;
				});
				
				return true;
			};
			
			// Removing the tooltip finally
			var destroy = function() {
				to = clearTimeout(to);
				clearTimeout(rto);
				if (cont && cont.length) {
					cont.off(ns);
					cont.remove();
				}
			};
			
			// Showing, aligning and fading in
			var show = function() {
				var o = self.offset();
				//log(o.left + ':' + o.top);
				if (o.top === offs.top && o.left === offs.left) {
					// only if target layer not in move
					cont.fadeIn(300).alignTo(self, {
						gap: 10,
						pos: settings.pos
					});
				}
			};
			
			// Entering the hotspot
			var enter = function() {
				
				if (!cont) {
					if (!create()) {
						return;
					}
				} else {
					cont.stop(true, false);
					rto = clearTimeout(rto);
				}
				
				offs = self.offset();
				//log(offs.left + ':' + offs.top);
				clearTimeout(to);
				to = setTimeout(show, settings.delay);
				
				if (!hasTouch) {
					over = true;
				} else {
					// Remove hint automatically on touch devices, because there's no explicit mouse leave event is triggered
					removeLater();
				}
			};
			
			// Touch (toggling the popup element)
			var touchStart = function(e) { //log('touchStart');
				e.preventDefault();
				//rto = clearTimeout(rto);
				if (!cont || cont.is(':hidden')) {
					enter();
				} else {
					over = false;
					leave();
					if (self[0].nodeName === 'A') {
						var l = self.attr('href');
						if (l && !l.startsWith('javascript:')) {
							//self[0].removeEventListener('click', click);
							window.location.href = l;
						} else {
							self.trigger('click');
						}
					}
				}
				return false;
			}
						
			var doNothing = function(e) {
				e.preventDefault();
				e.stopPropagation();
			};
			
			// Custom event = Force removing the hint
			self.on('removeTooltip', destroy);
			
			if (hasTouch) {
				self.on('touchstart' + ns + ' pointerdown' + ns + ' MSPointerDown' + ns, touchStart)
					.on('touchend' + ns + ' touchcancel' + ns + ' pointerup' + ns + ' MSPointerUp' + ns, doNothing)
					.on('click' + ns, doNothing);
			} else {
				self.on('focus' + ns + ' mouseenter' + ns, enter)
					.on('blur' + ns + ' mouseleave' + ns, leave)
					.on('click' + ns, clicked);
			}
		});
	};
	
	/*
		ALIGN_LEFT = ALIGN_TOP = 0
		ALIGN_CENTER = ALIGN_MIDDLE = 1
		ALIGN_RIGHT = ALIGN_BOTTOM = 2
	*/
	$.fn.addTooltip.defaults = {
		delay: 50,
		className: 'hint',
		nub: 'nub',
		stay: 2000,
		pos: [1,2,1,0]
	};
	
})(jQuery);
