;( function($, window, document, undefined) {
	'use strict';
	
	$.fn.lightbox = function(options) {
		
		options = $.extend({}, $.fn.lightbox.defaults, options);

		var text = getTranslations($.fn.lightbox.text),
			thumbs = $([]),
			thumb = $(),
			image = $(),
			media = $(),
			caption = $(),
			el,
			lightboxWidth,
			lightboxHeight,
			isImage = true,
			isVideo = false,
			isAudio = false,
			isOther = false,
			swiped = false,
			slideshow,
			fitTimeout,
			
			loadMediaFn,
			loadImageFn,

			ns = $(this).data('llb_ns'),
			btnclick = hasTouch? 'touchend' : clickEvent,
			extraSizes = options.hasOwnProperty('extraSizes')? options.extraSizes.split(/,\s*/) : false,

			cleanup = function(ns) {	
				// cleaning up the old event handlers
				$(window).add(document).off('.' + ns);
				// Removing the lightbox :: you can only have one lightbox per thumb set
				$('#' + ns).remove();
			},
			
			// Reacalculating image position and size upon window resize
			
			fitImage = function(initial) {
				
				var w, 
					h, 
					lbh,
					ratio, 
					img = image[0];
				
				clearTimeout(fitTimeout);
				
				if (!image || !image.length) {
					return;
				}
								
				if (!(w = image.data('oWidth'))) {
					w = (typeof img.naturalWidth === 'undefined')? (img.width || image.width()) : img.naturalWidth;
				}
				if (!(h = image.data('oHeight'))) {
					h = (typeof img.naturalHeight === 'undefined')? (img.height || image.height()) : img.naturalHeight;
				}
				
				if (w && h) {
				
					image.data({
						'oWidth': w,
						'oHeight': h
					});
					
					lbh = lightboxHeight - ((caption.length && options.fitAboveCaption)? caption.outerHeight() : 0);
					
					ratio = Math.min((lightboxWidth * options.fitRatio) / w, (lbh * options.fitRatio) / h);
	
					if (options.fitImages && !isOther) {
						if (ratio < 1 || options.scaleUp) {
							w *= ratio;
							h *= ratio;
						}
					}
					
					if (initial === true) {
						image.css({
							left: (lightboxWidth - w) / 2,
							top: (lbh - h) / 2,
							width: w,
							height: h
						}).show();
					} else {
						image.stop(true, false).animate({
							left: (lightboxWidth - w) / 2,
							top: (lbh - h) / 2,
							width: w,
							height: h
						}, 300, function() {
							$(this).translateX(0, 300)
						});
					}
					
					lb_zoom.toggleClass('active', ratio >= options.fitRatio && options.fitImages || (ratio < 1 && !options.fitImages))
						.toggleClass('invisible', isOther || ratio >= 1 && !options.scaleUp);
				
				} else if (initial) {
					// Wait for width and height get value
					fitTimeout = setTimeout(fitImage, true, 100);
				}

			},
			
			removeOldContent = function(direction) {
				
				var oldImage = lightbox.find('.lightbox-main'),
					oldCaption = lightbox.find('.lightbox-caption');
					
				if (oldImage.length) {
					
					// Removing the old image
					if (!swiped) {
						oldImage.translateXAndFade(-100 * direction, 0, options.speed, function() {
							$(this).remove();	
						});
					}
				}
				
				if (oldCaption.length) {
					
					// Removing the old caption
					oldCaption.find('.buttons a').trigger('removeTooltip');
					
					if (oldCaption.length > 1) {
						oldCaption.eq(-1).prevAll('.lightbox-caption').remove();
						oldCaption = lightbox.find('.lightbox-caption');
					}
					
					oldCaption.opacity(0, options.speed, function() {
						$(this).remove();	
					});
				}
			},
			
			loadImage = function(n) {
				
				if (typeof n !== 'number') {
					n = thumbs.index(n);
				}
				
				if (n < 0 || n > thumbs.length) {
					// Out of bounds move
					if (n < 0 && options.onBeforeFirst !== false) {
						options.onBeforeFirst(lightbox);
					} else if (options.onAfterLast !== false) {
						options.onAfterLast(lightbox);
					}
					quitLightbox();
					return;
				}
				
				var item,
					curr,
					direction = 0;
				
				// Direction is calculated if the current thumb exists
				if (lightbox.is(':visible')) {
					
					curr = thumbs.index(thumb);
					direction = (curr > n)? -1 : ((curr < n)? 1 : 0);
					
					// the previous image is on screen
					
					if (curr === n) {
						// the requested image is already on screen :: nothing to do
						return;
					}
					
				} else {
					$('body').css('overflow', 'hidden');
					lightbox.show();
				}
				
				lightbox.find('.lightbox-modal').fadeOut(function() {
					$(this).remove();
				});
				
				thumb = thumbs.eq(n);
				item = thumb.data(J.OBJ);

				lb_activity.show();
				lb_pause.find('.progress').hide();
				
				isVideo = item[J.CATEGORY] === 'video';
				isAudio = item[J.CATEGORY] === 'audio';
				isOther = item[J.CATEGORY] === 'other';
				isImage = !isVideo && !isAudio && !isOther;
				
				if (options.onLoadStart !== false) {
					options.onLoadStart(thumb);
				}
				
				setTimeout(function() {
					
					// Delayed image loading
										
					// Rejecting the previous unfinished image loads
					
					if (loadMediaFn && typeof loadMediaFn.state === 'function' && loadMediaFn.state() === 'pending') {
						//console.log('Cancelling old loadMedia');
						loadMediaFn.reject();
					}
					
					if (loadImageFn && typeof loadImageFn.state === 'function' && loadImageFn.state() === 'pending') {
						//console.log('Cancelling old loadImage');
						loadImageFn.reject();
					}
					
					if (isVideo) {
						
						// Video
						
						loadImageFn = null;
						loadMediaFn = $.Deferred();
						
						media = image = $('<video>', {
							'class': 'lightbox-main video',
							preload: 'auto',
							controls: 'true',
							poster: Album.getImagePath(item)
						})
						.on('loadedmetadata', function(e) {
							loadMediaFn.resolve(e);
						})
						.attr('src', Album.getSourcePath(item));
						
					} else if (isImage) {
						
						// Image
						
						loadImageFn = $.Deferred();
						loadMediaFn = null;
						
						media = image = $('<img>', {
							'class': 'lightbox-main image'
						})
						.on('load', function(e) {
							loadImageFn.resolve(e);
						})
						.hide()
						.attr('src', Album.getImagePath(item));
						
					} else {
						
						image = $('<div>', {
							'class': 'lightbox-main ' + item[J.CATEGORY]
						});
						
						if (isAudio) {
							
							// Audio
							
							loadMediaFn = $.Deferred();
							loadImageFn = $.Deferred();
							
							$('<img>', {
								'class': 'poster'
							}).on('load', function(e) {
								loadImageFn.resolve(e);
							})
							.attr('src', Album.getImagePath(item))
							.appendTo(image);
							
							media = $('<audio>', {
								preload: 'auto',
								controls: 'true'
							})
							.on('loadedmetadata', function(e) {
								loadMediaFn.resolve(e);
							})
							.attr('src', Album.getSourcePath(item))
							.appendTo(image);
							
						} else {
							
							// Other
							
							loadImageFn = $.Deferred();
							loadMediaFn = null;
							
							$('<a>', {
								href: Album.getSourcePath(item),
								target: '_blank'
							})
							.append($('<img>', {
								'class': 'other'
							})
							.on('load', function(e) {
								loadImageFn.resolve(e);
							})
							.attr('src', Album.getImagePath(item)))
							.appendTo(image);
							
							$('<p>', {
								'class': 'click-hint',
								'html': text.clickToOpen
							})
							.appendTo(image);
						}	
					}
					
					removeOldContent(direction);
					swiped = false;
					
					caption = $('<div>', {
						'class': 'lightbox-caption' + (slideshow? ' slideshow' : '')
					}).hide().appendTo(lightbox);
					
					var buttons = $('<div>', {
						'class': 'buttons'
					});

					if (extraSizes && item[J.CATEGORY] === 'image') {
						el = ($('<div>', {
							'class': 'download icon-download'
						})).appendTo(buttons);
						var p = Album.getImagePath(item);
						for (var i = 0; i < extraSizes.length; i++) {
							el.append($('<a>', {
								text: extraSizes[i],
								href:  p.replace('slides/', 'dl/' + extraSizes[i] + '/'),
								download: ''
							}));
						}
					}
					
					if (options.share) {
						buttons.renderShares({
							sites: options.share,
							title: item[J.TITLE] || item[J.NAME],
							description: item[J.COMMENT],
							image: Album.getAbsoluteImagePath(item),
							href: Album.getAbsolutePath(item)
						});
					}
										
					if (options.printImage && item[J.CATEGORY] === 'image') {
						el = $('<a>', {
							'class': 'secondary button small icon-printer',
							text: text.print
						}).appendTo(buttons);
						el.on(btnclick, function(e) {
							printImage((item[J.ORIGINAL] && !options.rightClickProtect)? item[J.ORIGINAL][J.PATH] : item[J.IMAGE][J.PATH],
								item[J.TITLE] || '',
								item[J.THUMBCAPTION] || ''
							);
						});
					}
				
					if (item[J.ORIGINAL] && !options.rightClickProtect && 
						(item[J.CATEGORY] === 'image' || options.allowDownloadOthers)) {
						buttons.append($('<a>', {
							'class': 'secondary button small icon-download',
							text: text.download,
							download: '',
							href: item[J.ORIGINAL][J.PATH]
						}));
					}
					
					if (options.shop) {
						el = $('<a>', {
							'class': 'secondary button small icon-shopping-cart',
							text: text.addCart
						}).appendTo(buttons);
						
						el.on(btnclick, function(e) {
							var s = $(options.shop.hook),
								btn = $(e.target);
								
							s.trigger('addItems', item);
							
							if (!$.cookie('shoppingCartInfo')) {
								var a,
									mod = $('<div>', {
										'class': 'lightbox-modal'
									}).appendTo(lightbox);
								
								mod.hide().fadeIn();
								
								mod.append($('<div>', {
									'class': 'shoppingcart-info',
									html: text.shoppingcartInfo
								}).append(a = $('<button>', {
									type: 'button',
									'class': 'lightbox-btn close',
									'title': 'Close'
								})));
								
								mod.add(a).on(btnclick, function(e) {
									if (e.target == this) {
										mod.fadeOut(function() {
											$(this).remove();
										});
										return false;
									}
								});
								
								$.cookie('shoppingCartInfo', true, 86400);
							}
							
							btn.addClass('disabled').text(text.added);
							setTimeout(function() {
								btn.removeClass('disabled').text(text.addCart);
							}, 1000);
						});
					}
					
					if (item[J.PHOTODATA]) {
						el = $('<a>', {
							'class': 'secondary button small icon-camera',
							text: text.photoData
						}).appendTo(buttons);
						
						el.on(btnclick, function() {
							var a,
								mod = $('<div>', {
									'class': 'lightbox-modal'
								}).appendTo(lightbox);
								
							mod.hide().fadeIn();
							
							mod.append($('<div>', {
								'class': 'photodata',
								html: item[J.PHOTODATA]
							}).append(a = $('<button>', {
								type: 'button',
								'class': 'lightbox-btn close',
								'title': 'Close'
							})));
							
							mod.add(a).on(btnclick, function(e) {
								if (e.target == this) {
									mod.fadeOut(function() {
										$(this).remove();
									});
									return false;
								}
							});
							
							return false;
						});
					}
					
					if (item[J.LOCATION]) {
						el = $('<a>', {
							'class': 'secondary button small icon-location',
							text: text.map
						}).appendTo(buttons);
						
						el.on(btnclick, function() {
							var a,
								target,
								mod = $('<div>', {
									'class': 'lightbox-modal'
								}).appendTo(lightbox);
								
							mod.hide().fadeIn();
							
							mod.append(target = $('<div>', {
								'class': 'map'								
							}).append(a = $('<button>', {
								type: 'button',
								'class': 'lightbox-btn close',
								'title': 'Close'
							})));
							
							mod.add(a).on(btnclick, function(e) { 
								if (e.target == this) {
									mod.fadeOut(function() {
										$(this).remove();
									});
									return false;
								}
							});
							
							target = $('<div>', {
								'class': 'map-cont'
							}).height($(window).height() * 0.6).appendTo(target);
							
							target.data('location', item[J.LOCATION]);
							
							if (options.mapType) {
								target.data('map-type', options.mapType);
							}
							
							if (options.mapZoom) {
								target.data('map-zoom', options.mapZoom);
							}
							
							setTimeout(function() {
								loadMap(target);
							}, 100);
							
							return false;
						});
					}
					
					if (item[J.MOSTPHOTOS]) {
						buttons.append($('<a>', {
							'class': 'secondary button small icon-shopping-cart',
							text: text.buyImage,
							target: '_blank',
							href: 'http://mostphotos.com/' + item[J.MOSTPHOTOS]
						}));
					}
					
					if (!buttons.is(':empty')) {
						caption.addClass('hasbuttons');
						caption.append(buttons);
					}
					
					if (item[J.IMAGECAPTION]) {
						caption.append($('<div>', {
							'class': 'caption',
							html: item[J.IMAGECAPTION]
						}));
					}
					
					caption.toggle(!slideshow || caption.children('.caption').length > 0);
					
					// Ready to rock
					
					$.when(loadImageFn, loadMediaFn).done(function(e) {
						
						if (typeof image[0] === 'undefined' || typeof media[0] === 'undefined') {
							console.log('Premature ' + ((typeof image[0] === 'undefined')? 'loadImage':'loadMedia') + '.done(' + Album.getImagePath(item) + ')');
						}
						
						image.css({
							opacity: 0
						}).hide().appendTo(lightbox);
						
						if (options.rightClickProtect) {
							media.on('contextmenu', function(e) {
								e.preventDefault()
								return false;
							});
						}
													
						if (isVideo || isAudio) {
							media.attr('autoplay', options.videoAuto);
							media[0].volume = options.volume;
							media[0].onvolumechange = function() {
								options.volume = this.volume;
							};
						}

						// Moving away from center
						image.translateX(100 * direction);
						
						// Leaving time for CSS transform to settle down
						setTimeout(function() {
								
							fitImage(true);
							
							lb_activity.hide();
							// Moving back to center
							image.show().translateXAndFade(0, 1, options.speed);
							
							// Preparing the image after transition
							setTimeout(function() {
									
								if (hasTouch && !(options.fitAboveCaption && options.fitImages)) {
									setTimeout(function() {
										hideControls();
										hideCaption();
									}, slideshow? Math.min(options.slideshowDelay / 2, 2000) : 2000);
								}
								
								
								// Toggling caption / video controls on touch
								image.on(hasTouch? 
									('touchend.' + ns + ' touchcancel.' + ns + (pointerUpEvent? (' ' + pointerUpEvent + '.' + ns) : ''))
									:
									('click.' + ns), function( e ) {
									
									/*
									if ((isVideo || isAudio) && (e.offsetY || e.originalEvent.offsetY) < (image.height() - 40)) {
										return true;
									}
									*/
										
									setTimeout(function() {
										// delaying a bit to allow swipe event finished if it's on
										if (!swiped) {
											if (isVideo || isAudio) {
												var m = media[0];
												if (m.paused) {
													m.play();
													m.controls = false;
													hideCaption();
													if (hasTouch) {
														hideControls();
													}
												} else {
													m.pause();
													m.controls = true;
													showCaption();
													if (hasTouch) {
														showControls();
													}
												}
											} else if (isImage) {
												if (hasTouch) {
													if (!caption.is(':visible') || caption.css('opacity') < 1) {
														showControls();
														showCaption();
													} else {
														hideControls();
														hideCaption();
													}
												} else {
													// image navigation
													if (((e.pageX || e.originalEvent.pageX) - image.position().left) > (image.width() / 2)) {
														nextImage();
													} else {
														previousImage();
													}
												}
											}
										}
									}, hasTouch? 50 : 20);
									
									return isOther || isVideo || isAudio;
								});
							
								if (hasTouch || !(isAudio || isVideo)) { 
									// Swipe handler
									image.swipe({
										onSwipedLeft: function() {
											swiped = true;
											nextImage();
										},
										onSwipedRight: function() {
											swiped = true;
											previousImage();
										},
										onFinished: function() {
											$(this).trigger('removeSwipe');
											$(this).remove();
										}
									});
								}

								if (options.onLoadEnd !== false) {
									options.onLoadEnd(thumb);
								}
							}, options.speed);
						}, 50);
												
						// Preloading next image
						if (options.preloadNext) {
							var nextItem;
							if (direction < 0 && n > 0) {
								nextItem = thumbs.eq(n - 1).data(J.OBJ);
							} else if (n < thumbs.length - 2) {
								nextItem = thumbs.eq(n + 1).data(J.OBJ);
							}
							if (nextItem) {
								if (nextItem[J.CATEGORY] === 'image') {
									$('<img>', {
										'src': Album.getImagePath(nextItem)
									}).load();
								}
							}
						}
						
						// Continuing slideshow
						if (slideshow) {
							if (isAudio || isVideo) {
								suspendAuto();
								media[0].onended = function() {
									resumeAuto();
								};
							} else {
								clearTimeout(slideshow);
								lb_pause.find('.progress').show();
								slideshow = setTimeout(nextImage, options.slideshowDelay);
							}
						} else {
							stopAuto();
						}
					})
					.fail(function() {
						if (options.onLoadEnd !== false) {
							options.onLoadEnd(thumb);
						}
					});
										
				}, hasTouch? options.speed + 50 : options.speed / 3);
			},
			
			// Hiding the lightbox overlay
			quitLightbox = function() {
				stopAuto();
				$('body').css('overflow', '');
				if (!image.length) {
					if (options.onEnd !== false) {
						options.onEnd(thumb);
					}
				} else {
					lightbox.fadeOut(options.speed, function() {
						removeImage();
						//inProgress = false;
						if (options.onEnd !== false) {
							options.onEnd(thumb);
						}
					});
				}
			},
			
			// Removing the lightbox completely
			removeLightbox = function() {
				stopAuto();
				// removing event handlers
				thumbs.off('.' + ns);
				$(document).add($(window)).add(lightbox).off('.' + ns);
				$('body').css('overflow', '');
				if (!image.length) {
					// No image
					lightbox.remove();
				} else {
					// Fade out
					image.animate({ 
						'opacity': 0 
					}, options.speed, function() {
						lightbox.remove();
					});
				}
			},
			
			removeImage = function() {
				if (image.length) {
					image.remove();
					image = $();
				}
				if (caption.length) {
					caption.remove();
					caption = $();
				}
			},
			
			startAuto = function() {
				clearTimeout(slideshow);
				lb_play.hide();
				lb_pause.show();
				lb_btn_left.add(lb_btn_right).add(lb_close).add(lb_zoom).fadeOut(1000);
				caption.addClass('slideshow');
				slideshow = setTimeout(nextImage, options.slideshowDelay / 4);
				if (options.onSlideshowStart !== false) {
					options.onSlideshowStart(thumb);
				}
			},
			
			resumeAuto = function() {
				slideshow = setTimeout(nextImage, options.slideshowDelay / 4);
				lb_pause.find('.progress').show();
			},
	
			stopAuto = function() {
				if (options.onSlideshowPause !== false) {
					options.onSlideshowPause(thumb);
				}
				slideshow = clearTimeout(slideshow);
				lb_play.show();
				lb_pause.hide();
				lb_close.show();
				lb_btn_left.add(lb_btn_right).add(lb_zoom).show();
				caption.show().removeClass('slideshow');
			},
			
			suspendAuto = function() {
				slideshow = clearTimeout(slideshow);
				lb_pause.find('.progress').hide();
			},
			
			toggleZoom = function() {
				options.fitImages = !options.fitImages;
				fitImage();
			},
			
			showCaption = function() {
				caption.show();
			},
			
			hideCaption = function() {
				caption.fadeOut(1000);
			},
			
			showControls = function() {
				lb_close.add(lb_zoom).show();
			},
			
			hideControls = function() {
				lb_close.add(lb_zoom).fadeOut(1000);
			},
			
			showArrows = function() {
				lb_btn_left.add(lb_btn_right).show();
			},
			
			hideArrows = function() {
				lb_btn_left.add(lb_btn_right).fadeOut(1000);
			},
			
			previousImage = function() {
				var curr = thumbs.index(thumb);
				if (curr > 0) {
					loadImage(curr - 1);
				} else {
					if (options.onBeforeFirst !== false) {
						options.onBeforeFirst();
					}
					quitLightbox();
				}
			},
			
			nextImage = function() {
				var curr = thumbs.index(thumb);
				if (curr < thumbs.length - 1) {
					loadImage(curr + 1);
				} else {
					if (options.onAfterLast !== false) {
						options.onAfterLast();
					}
					quitLightbox();
				}
			};
			
		// Initializing Lightbox
					
		if (ns !== undefined) {
			// should never happen, but still living lightbox attached to thumb elements
			cleanup(ns);
		}
		
		// Creating new namespace
		$(this).data('llb_ns', ns = 'llb_' + Math.floor(Math.random() * 10000));		

		// Finding thumbs
		thumbs = $(this).find(options.delegate);
		
		// Setting up the structure
				
		var lightbox = $('<div>', {
				id: ns,
				'class': 'lightbox'
			}).hide().appendTo('body'),
			
			// Darken background
			lb_overlay = $('<div>', {
				'class': 'lightbox-overlay'
			}).appendTo(lightbox),
			
			// Activity indicator
			lb_activity = $('<div>', {
				'class': 'lightbox-loading'
			}).append('<div>').appendTo(lightbox),
			
			// Close button
			lb_close = $('<button>', {
				type: 'button',
				'class': 'lightbox-btn close',
				'title': text.close
			})
			.on(btnclick, function(){ 
				quitLightbox(); 
				return false; 
			})
			.toggle(!hasTouch)
			.appendTo(lightbox),
			
			// Zoom toggle button
			lb_zoom = $('<button>', {
				type: 'button',
				'class': 'lightbox-icon zoom icon-expand',
				'title': text.toggleZoom
			})
			.on(btnclick, function(){
				toggleZoom(); 
				return false; 
			})
			.appendTo(lightbox),
			
			// Play button
			lb_play = $('<button>', {
				type: 'button',
				'class': 'lightbox-icon play icon-play',
				'title': text.slideshow
			})
			.on(btnclick, function(){ 
				startAuto(); 
				return false; 
			})
			.appendTo(lightbox),

			// Pause button
			lb_pause = $('<button>', {
				type: 'button',
				'class': 'lightbox-icon pause icon-pause',
				'title': text.pause
			})
			.append($('<span>', {
				'class': 'progress'
			}))
			.hide()
			.on(btnclick, function(){ 
				stopAuto(); 
				return false; 
			})
			.appendTo(lightbox),
			
			// Buttons
			lb_btn_left = (!hasTouch && thumbs.length)?
				$('<button>', {
					type: 'button',
					'class': 'lightbox-btn arrow-left'
				})
				.on(btnclick, function(){
					previousImage();
					return false;
				})
				.appendTo(lightbox)
				: $(),
				
			lb_btn_right = (!hasTouch && thumbs.length)?
				$('<button>', {
					type: 'button',
					'class': 'lightbox-btn arrow-right'
				})
				.on(btnclick, function(){
					nextImage();
					return false;
				})
				.appendTo(lightbox)
				: $();

		// Setting up events
				
		// Resize
		var resizeTimeout = null;
		lightboxWidth = lightbox.width();
		lightboxHeight = lightbox.height();
		$(window).on('resize.' + ns + ' orientationchange.' + ns, function() {
			clearTimeout(resizeTimeout);
			// Executed only if 50ms has passed since the last resize event
			resizeTimeout = setTimeout(function() { 
				lightboxWidth = lightbox.width();
				lightboxHeight = lightbox.height();
				if (image && image.length) {
					fitImage();
				}
			}, 50);
		});
		
		// Quit on document touch
		if (options.quitOnDocClick) {
			lb_overlay.on(hasTouch? ('touchend.' + ns) : ('click.' + ns), function( e ) {
				if (image.length && !$(e.target).is(image)) {
					quitLightbox();
				}
			});
		}
		
		// Keyboard
		if (options.enableKeyboard) {
			$(document).on('keyup.' + ns, function( e ) {
				if (!image.length) {
					return true;
				}
				
				e.preventDefault();
				
				switch (e.keyCode) {
					
					case 27:
						quitLightbox();
						break;
					
					case 37:
						previousImage();
						break;
						
					case 39:
						nextImage();
						break;
						
					case 107:
						toggleZoom();
						break;
						
					case 32:
						if (isVideo || isAudio) {
							if (media[0].paused) {
								media[0].play();
							} else {
								media[0].pause();
							}
						} else {
							if (slideshow) {
								stopAuto();
							} else {
								startAuto();
							}
						}
						break;
						
					default:
						return true;
										
				}
				
				return false;
			});
		}
		
		// Mouse wheel
		if (options.enableMouseWheel) {
			var wheelTimeout = null;
			$(window).on('mousewheel.' + ns + ' DOMMouseScroll.' + ns, function(event) {
				wheelTimeout = clearTimeout(wheelTimeout);
				if (!image.length) {
					return true;
				}
				wheelTimeout = setTimeout((event.originalEvent.wheelDelta > 0 || event.originalEvent.detail < 0)? previousImage : nextImage, 100); 
				return false;
			});
		}
		
		// Events that can be triggered from outside
		
		// External trigger to switch image
		$(this).on('lightboxLoad.' + ns, function(e, n) {
			stopAuto();
			if (options.onStart !== false) {
				options.onStart((typeof n === 'number')? thumbs.eq(n) : th);
			}
			loadImage(n);
		})
		// External trigger to quit = does not remove, just hides the lightbox
		.on('lightboxQuit.' + ns, function() {
			quitLightbox();
			return false;
		})
		// Removing the lightbox completely
		.on('lightboxRemove.' + ns, function() {
			removeLightbox();
			return false;
		})
		// Returning the current lightbox container
		.on('lightboxContainer.' + ns, function() {
			return lightbox;
		})
		// External trigger to go to the previous image
		.on('lightboxNext.' + ns, function() {
			stopAuto();
			nextImage();
			return false;
		})
		// External trigger to go to next image
		.on('lightboxPrevious.' + ns, function() {
			stopAuto();
			previousImage();
			return false;
		});

		// Start
		if (options.auto !== undefined && $.isNumeric(options.auto)) {
			if (options.auto < 0) {
				options.auto = thumbs.length + options.auto;
			}
			if (options.onStart !== false) {
				options.onStart(thumbs.eq(options.auto));
			}
			loadImage(options.auto);
		}
		
		// Ready event
		
		if (options.onReady) {
			options.onReady(thumb);
		}

		return this;
	};
	
	$.fn.lightbox.defaults = {
		delegate:			'li',
		auto:				false,
		slideshowDelay:		4000,
		fitImages:			false,
		scaleUp:			true,
		fitRatio:			0.94,
		fitAboveCaption:	true,
		videoAuto:			false,
		volume:				0.5,
		rightClickProtect: 	false,
		allowDownloadOthers:true,
		printImage:			false,
		speed: 				400,
		preloadNext:		true,
		enableKeyboard: 	true,
		enableMouseWheel:	true,
		quitOnEnd:			true,
		quitOnDocClick: 	true,
		onStart:			false,
		onEnd:				false,
		onLoadStart:		false,
		onLoadEnd:			false,
		onAfterLast:		false,
		onBeforeFirst:		false,
		onReady:			false,
		onSlideshowStart:	false,
		onSlideshowPause:	false
	};
	
	$.fn.lightbox.text = {
		close:				'Close',
		slideshow:			'Slideshow',
		pause:				'Pause',
		toggleZoom:			'Toggle zoom',
		download: 			'Download',
		print:				'Print',
		map: 				'Map',
		buyImage: 			'Buy image',
		photoData: 			'Photo data',
		addCart:			'Add to cart',
		added:				'Added...',
		shoppingcartInfo:	'This item is placed into the "Shopping cart". You can buy it by clicking the "Buy now" button after you quit the lightbox.', 
		clickToOpen:		'Click to open this document with the associated viewer!'
	};

	
})(jQuery, window, document);
