;( function( $, window, document, undefined ) {
	'use strict';
	/* 
		Swipe action using CSS transforms with jQuery position fallback
	*/
	
	$.fn.swipe = function(options) {
		
		options = $.extend({
				onSwipedLeft:	false,
				onSwipedRight:	false,
				onSmallSwipe:	false,
				onFinished:		false,
				treshold:		50,
				swipeoutSpeed:	300
			}, options);

		var cleanup = function(ns) {	
				// cleaning up the old event handlers
				if (ns) {
					$(window).add(document).off('.' + ns);
				}
			};
		
		return this.each(function() {
				
			var self = $(this),
				ns = $(this).data('lsw_ns'),
				swipeStarted,
				swipeStart = 0,
				swipeDiff = 0,
				x0 = 0,
				timer = null,
				
				// Start moving
				swipeStart = function(e) {
					//log('Touch start, fingers: ' + (e.originalEvent.touches? e.originalEvent.touches.length : 0));
					if (e.originalEvent.touches && e.originalEvent.touches.length > 1) {
						// multi finger touch
						return true;
					}
					self.stop(true, false);
					swipeStarted = new Date().getTime();
					swipeStart = e.originalEvent.touches? e.originalEvent.touches[0].pageX : e.originalEvent.pageX;
					x0 = self.translateX();
					self.on('mousemove.' + ns, swipeMove);
					return false;
				},
				
				// Moving in progress
				swipeMove = function(e) {
					//log('Touch move, fingers: ' + (e.originalEvent.touches? e.originalEvent.touches.length : 0));
					if (e.originalEvent.touches && e.originalEvent.touches.length > 1) {
						// multi finger touch
						return true;
					}
					swipeDiff = (e.originalEvent.touches? e.originalEvent.touches[0].pageX : e.originalEvent.pageX) - swipeStart;				
					self.translateX(x0 + swipeDiff);
					return false;
				},
				
				// Stopped moving
				swipeEnd = function(e) {
					e.preventDefault();
					$(this).off('mousemove.' + ns);
					if (Math.abs(swipeDiff) > options.treshold) {
						var dt = new Date().getTime() - swipeStarted,
							targetPos = x0 + swipeDiff * (1 + options.swipeoutSpeed / dt);
							
						if ((swipeDiff < 0)? (self.position().left + self.width()) < self.parent().width() : (self.position().left > 0)) {
							// Swipe action
							self.translateXAndFade(targetPos, 0, options.swipeoutSpeed);
							
							timer = setTimeout(function() {
								if (typeof options.onFinished === 'function') {
									options.onFinished.call(self[0]);
								}
							}, options.swipeoutSpeed + 20);
							
							if (swipeDiff > 0) {
								// Right
								if (typeof options.onSwipedRight === 'function') {
									options.onSwipedRight.call(self[0]);
								}
							} else {
								// Left
								if (typeof options.onSwipedLeft === 'function') {
									options.onSwipedLeft.call(self[0]);
								}
							}
						} else {
							// Hasn't reached the edge yet
							self.translateX(targetPos, options.swipeoutSpeed);
						}
						
					} else {
						// Small swipe == click
						self.translateX(0, 100);
						if (typeof options.onSmallSwipe === 'function') {
							options.onSmallSwipe.call(self[0]);
						}
					}
					return false;
				};
				
			// clean up events that may hang around (in theory this should never happen)
			cleanup(ns);
			
			// Creating new Unique ID
			$(this).data('lsw_ns', 'lsw_' + Math.floor(Math.random() * 10000));
			
			self.on('touchstart.' + ns + (pointerDownEvent? (' ' + pointerDownEvent + '.' + ns) : '') + ' dragstart.' + ns, swipeStart)
			.on('touchmove.' + ns + (pointerMoveEvent? (' ' + pointerMoveEvent + '.' + ns) : ''), swipeMove)
			.on('touchend.' + ns + ' touchcancel.' + ns + (pointerUpEvent? (' ' + pointerUpEvent + '.' + ns) : '') + ' mouseup.' + ns + ' dragcancel.' + ns, swipeEnd)
				// External remove request
				.on('removeSwipe.' + ns, function() {
					self.off('.' + ns);
					cleanup(ns);
				});
	
		});
	};
			
})(jQuery, window, document);
