/*
 * laza.util - miscellaneous utility functions and prototype extensions
 *
 */

// Ensuring no 'console is undefined' errors happen

window.console = window.console || (function(){
	var c = {}; c.log = c.warn = c.debug = c.info = c.error = c.time = c.dir = c.profile = c.clear = c.exception = c.trace = c.assert = function(s){};
	return c;
})();

/*
 *	New constants
 */

var UNDEF = 'undefined',
	NOLINK = 'javascript:void(0)',
	LOCAL = location.protocol.indexOf('file:') === 0;
	
var VEND = (function(ua) {
/*
	PC:
		IE 8: "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0)"
		IE 9: "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Tablet PC 2.0; .NET4.0C)"
		IE 10: "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0; .NET4.0E; .NET4.0C)"
		Opera 12: "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.15"
		Firefox 21: "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/20100101 Firefox/21.0"
		Chrome 27: "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.94 Safari/537.36"
	Mac:
		Chrome 27: "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36"
		Firefox 21: "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:21.0) Gecko/20100101 Firefox/21.0"
		Safari 6: "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/536.26.14 (KHTML, like Gecko) Version/6.0.1 Safari/536.26.14"
 */
		if (ua.indexOf('Trident') > 0) {
			return 'ms';
		} else if (ua.indexOf('AppleWebKit') > 0) {
			return 'webkit';
		} else if (ua.indexOf('Gecko') > 0) {
			return 'moz';
		} else if (ua.indexOf('Presto') > 0) {
			return 'o';
		} else if (ua.indexOf('Blink') > 0) {
			return 'webkit';
		}
		return '';		
	}(navigator.userAgent));

/*
 *	Extending prototypes
 */
 
String.wsp = [];
String.wsp[0x0009] = true;
String.wsp[0x000a] = true;
String.wsp[0x000b] = true;
String.wsp[0x000c] = true;
String.wsp[0x000d] = true;
String.wsp[0x0020] = true;
String.wsp[0x0085] = true;
String.wsp[0x00a0] = true;
String.wsp[0x1680] = true;
String.wsp[0x180e] = true;
String.wsp[0x2000] = true;
String.wsp[0x2001] = true;
String.wsp[0x2002] = true;
String.wsp[0x2003] = true;
String.wsp[0x2004] = true;
String.wsp[0x2005] = true;
String.wsp[0x2006] = true;
String.wsp[0x2007] = true;
String.wsp[0x2008] = true;
String.wsp[0x2009] = true;
String.wsp[0x200a] = true;
String.wsp[0x200b] = true;
String.wsp[0x2028] = true;
String.wsp[0x2029] = true;
String.wsp[0x202f] = true;
String.wsp[0x205f] = true;
String.wsp[0x3000] = true;

String.prototype.trim = function() { 
	var str = this + '', j = str.length;
	if (j) {
		var ws = String.wsp, 
			i = 0;
		--j;
		while (j >= 0 && ws[str.charCodeAt(j)]) {
			--j;
		}
		++j;
		while (i < j && ws[str.charCodeAt(i)]) { 
			++i; 
		}
		str = str.substring(i, j);
	}
	return str;
};

String.prototype.trunc = function( n ) {
	var t = this + '';
	if (t.length <= n) {
		return t.toString();
	}
	var s = t.substring(0, n - 1), i = s.lastIndexOf(' ');
	return ((i > 6 && (s.length - i) < 20)? s.substring(0, i) : s) + '...';
};

String.prototype.startsWith = function( s ) {
	return (this + '').indexOf( s ) === 0;
};

String.prototype.endsWith = function( s ) {
	return (this + '').substring(this.length - s.length) === s;
};

String.prototype.capitalize = function() {
	return this.charAt(0).toUpperCase() + this.slice(1);
};

String.prototype.unCamelCase = function() {
	return this.replace(/([a-z])([A-Z])/g, "$1-$2").toLowerCase();
}
String.prototype.getExt = function() {
	var t = this + '', i = t.lastIndexOf('.');
	return (i <= 0 || i >= t.length - 1)? '' : t.substring(i + 1).toLowerCase();
};

String.prototype.hasExt = function(x) {
	var t = (this + ''), i = t.lastIndexOf('.');
	if (i >= 0) {
		t = t.substring(i + 1).toLowerCase();
		return (x + ',').indexOf(t + ',') >= 0;
	}
	return !1;
};

String.prototype.replaceExt = function( s ) {
	var t = this + '', i = t.lastIndexOf('.');
	return (i <= 0)? t : (t.substring(0, i + 1) + s);  
};

String.prototype.fixExtension = function() {
	return (this + '').replace(/.gif$/gi, '.png').replace(/.tif+$/gi, '.jpg');
};

String.prototype.getDir = function() {
	var u = (this + '').split('#')[0];
	return u.substring(0, u.lastIndexOf('/') + 1);
};

String.prototype.getFile = function() {
	var u = (this + '').split('#')[0];
	return u.substring(u.lastIndexOf('/') + 1);
};

String.prototype.getRelpath = function(level) {
	var t = (this + ''), i = t.lastIndexOf('#');
	if (i === -1) {
		i = t.length - 1;
	} else {
		i--;
	}
	for (; i >= 0; i--) {
		if (t[i] === '/' && (level--) === 0)
			break;
	}
	return t.substring(i + 1);
};

String.prototype.fixUrl = function() {
	var i, j, t = this + '';
	while ((i = t.indexOf('../')) > 0) {
		if (i === 1 || (j = t.lastIndexOf('/', i - 2)) === -1) {
			return t.substring(i + 3);
		}
		t = t.substring(0, j) + t.substring(i + 2);
	}
	return t;
};

String.prototype.fullUrl = function() {
	var t = this + '';
	if (!t.match(/^(http|ftp|file)/)) {
		t = window.location.href.getDir() + t;
	}
	return t.fixUrl();
};

String.prototype.cleanupHTML = function() {
	var htmlregex = [
		[ /<br>/gi, '\n' ],
		[ /\&amp;/gi, '&' ],
		[ /\&lt;/gi, '<' ],
		[ /\&gt;/gi, '>' ],
		[ /\&(m|n)dash;/gi , '-' ],
		[ /\&apos;/gi, '\'' ],
		[ /\&quot;/gi, '"' ]
	];
	var t = this + '';
	for ( var i = htmlregex.length - 1; i >= 0; i-- ) {
		t = t.replace( htmlregex[i][0], htmlregex[i][1] );
	}
	return t; 
};

String.prototype.stripHTML = function() { 
	return (this + '').replace(/<br\/?>/gi, '/ ').replace(/<\/[^>]+>/gi, ' ').replace(/<[^>]+>/gi, '').trim(); 
};

String.prototype.stripQuote = function() {
	return (this + '').replace(/\"/gi, '&quot;');
};

String.prototype.appendSep = function(s, sep) {
	return (this.length? (this + (sep || ' &middot; ')) : '') + s; 
};

String.prototype.rgb2hex = function() {
	var t = this + '';
	if (t.charAt(0) === '#' || t === 'transparent') {
		return t;
	}
	var n, r = t.match(/\d+/g), h = '';
	if ( r ) {
		for ( var i = 0; i < r.length && i < 3; i++ ) {
			n = parseInt( r[i], 10 ).toString(16);
			h += ((n.length < 2)? '0' : '') + n;
		}
		return '#' + h;
	}
	return 'transparent';
};

String.prototype.template = function( s ) {
	if ( typeof s === UNDEF || !this ) {
		return this;
	}
	if ( !isNaN(parseFloat(s)) && isFinite(s) ) {
		s = s + '';
	}
	var t = this + '';
	if ( s.constructor === Array ) {
		for ( var i = 0; i < s.length; i++ ) {
			t = t.replace( new RegExp('\\{' + i + '\\}', 'gi'), s[i] );
		}
	} else {
		t = t.replace( /\{0\}/gi, s );
	}
	return t;
};

String.prototype.getSearchTerms = function() {
	var t = this + '';
	
	if (t.indexOf('"') === -1) {
		return t.split(' ');
	} else {
		var a = [],
			i;
	
		do {
			if ((i = t.indexOf('"')) > 0) {
				a.push.apply(a, t.substring(0, i).split(' '));
			}
			t = t.substring(i + 1);
			i = t.indexOf('"');
			if (i < 0) {
				a.push(t);
				break;
			}
			a.push(t.substring(0, i));
			t = t.substring(i + 1);
			
		} while (t.length);
	
		return a;
	}
};

String.prototype.objectify = function() {
	if (!this || !this.length) {
		return this;
	}
	var t = this + '';
	if (t.charAt(0) === '?' || t.charAt(0) === '#') {
		t = t.substring(1);
	}
	var r = {}, o, os = t.split('&');
	for (var i = 0, l = os.length; i < l; i++) {
		o = os[i].split('=');
		if (o.length > 1) {
			r[o[0]] = decodeURIComponent(o[1]);
		}
	}
	return r;
};

// Test if 's' is in the string
String.prototype.testIn = function(s) {
	if (typeof s !== 'string') {
		s = s + '';
	}
	return (new RegExp(this, 'i')).test(s);
};

// Testing exact match
String.prototype.testExactMatch = function(s) {
	if (s.constructor !== Array) {
		return this == s + '';
	} else {
		for (var i = 0, l = s.length; i < l; i++) {
			if (this == s[i]) {
				return true;
			}
		}
	}
	return false;
};

// Ignore case but exact match
String.prototype.testMatch = function(s) {
	var t = this.toLowerCase(); 
	if (s.constructor !== Array) {
		return t == (s + '').toLowerCase();
	} else {
		for (var i = 0, l = s.length; i < l; i++) {
			if (t == s[i].toLowerCase()) {
				return true;
			}
		}
	}
	return false;
};

Math.minMax = function(a, b, c) {
	b = (isNaN(b))? parseFloat(b) : b;
	return  (b < a)? a : ((b > c)? c : b); 
};

var isEmpty = function(o) {
		if (o == null) {
			return true;
		};
		for (var i in o) { 
			return false; 
		} 
		return true; 
	},
	
	paramize = function(o) {
		if (typeof o === 'number') {
			return '' + o;
		} else if (typeof o === 'string') {
			return o;
		} else if (typeof o === 'object') {
			// 1 level depth only
			var s = ''; 
			for (var key in o) {
				if (o[key] !== null) {
					s += '&' + key + '=' + encodeURIComponent(o[key]);
				}
			}
			if (s.length)
				return s.substring(1);
		}
		return '';
	};

/*
 *	New functions and variables for the global context
 */

var getCoords = function(e) {
		var x, y;
		
		if (!e.touches) {
			// No touch
			return { 
				x: e.clientX, 
				y: e.clientY 
			};
		} else if (e.touches.length == 1) {
			return {
				x: e.touches[0].clientX,
				y: e.touches[0].clientY
			}
		} else if (e.changedTouches && e.changedTouches.length == 1) {
			return {
				x: e.changedTouches[0].clientX,
				y: e.changedTouches[0].clientY
			}
		}
		return null;
	},
		
	hasTouch = ('ontouchstart' in window),
	hasPointers = window.navigator.pointerEnabled || window.navigator.msPointerEnabled,
	pointerDownEvent = hasPointers? (window.navigator.pointerEnabled? 'pointerdown' : 'MSPointerDown') : null,
	pointerMoveEvent = hasPointers? (window.navigator.pointerEnabled? 'pointermove' : 'MSPointerMove') : null,
	pointerUpEvent = hasPointers? (window.navigator.pointerEnabled? 'pointerup' : 'MSPointerUp') : null,
	clickEvent = pointerUpEvent || 'click',

	
	wasTouched = function(event) {
		if (hasTouch) {
			// Normal browsers
			return true;
		} else if (hasPointers) {
			// IE
			if (typeof event === undefined || !event.hasOwnProperty('pointerType')) {
				return false;
			} else if (event.pointerType === 'touch') {
				// Win 8.1+ IE 11+
				return true;
			} else {
				// IE 10 :: not mouse == touch or pen
				return event.pointerType !== 0x00000004
			}
		}
		// Unknown device
		return false;
	},

	// Dummy function to avoid further events
	
	noAction = function(e) {
		e.stopPropagation();
		e.preventDefault();
		return false;
	},


	/*
	 *	Translate function
	 *	returns a translated key, or the default value (if exists), or the key itself de-camelcased
	 */
	
	translate = function(key, def) {
		
		key = key.trim();
		
		if (typeof Texts !== UNDEF) {
			if (Texts.hasOwnProperty(key)) {
				return Texts[key];
			}
		}
		
		if (typeof def !== UNDEF) {
			// Using the default
			if (DEBUG && console) {
				console.log('Using default translation: '+key+'='+def);
			}
			return def;
		}
		
		if (DEBUG && console) {
			console.log('Missing translation: '+key);
		}
		
		var s = key.replace(/([A-Z])/g, ' $1').toLowerCase();
		
		s[0] = s.charAt(0).toUpperCase();
		return s;
	},
	
	/* 
	 *	Simpler method 
	 *	text = getKeys('key1,key2,key3', [defaults])
	 */
	
	getKeys = function(keys, def) {
		
		var t = {}, i, k = keys.split(','), l = k.length;
		
		for (i = 0; i < l; i++) {
			t[k[i]] = translate(k[i], def[k]);
		}
		
		return t;
	},
	
	/*
	 *	Finds translation for each key in def Object
	 *	def should contain only elements that need translation
	 */
	
	getTranslations = function(def) {
		
		var t = {}, k;
		
		for (k in def) {
			if (typeof def[k] === 'object') {
				t[k] = getTranslations(def[k]);
			} else {
				t[k] = translate(k, def[k]);
			}
		}
		
		return t;
	},
	
	/*
	 *  Relative date
	 */
	
	getRelativeDate = function(days) {
		
		if (!days)
			return translate('today');
		
		if (days===1)
			return translate('yesterday');
			
		var s, n;
		
		if (days >= 730) {
			s = translate('yearsAgo');
			n = Math.round(days / 365);
		} else if (days >= 60) {
			s = translate('monthsAgo');
			n = Math.round(days / 30.5);
		} else {
			s = translate('daysAgo');
			n = days;
		}
		
		return s.replace('{0}', n);
	},

	/*
	 *  Timespan
	 */
	
	getTimespan = function(days) {
		
		if (!days)
			return translate('today');
		
		if (days===1)
			return translate('yesterday');
			
		var s, n;
		
		if (days >= 730) {
			s = translate('inThePastNYears');
			n = Math.round(days / 365);
		} else if (days >= 60) {
			s = translate('inThePastNMonths');
			n = Math.round(days / 30.42);
		} else {
			s = translate('inThePastNDays');
			n = days;
		}
		
		return s.replace('{0}', n);
	},

	
	/* 
	 *	Pure JS extend function
	 */
	
	extend = function() {
		for (var i = 1; i < arguments.length; i++)
			for (var key in arguments[i])
				if (arguments[i].hasOwnProperty(key))
					arguments[0][key] = arguments[i][key];
		return arguments[0];
	},
	
	/*
	 *	Passing defaults to libraries
	 */
	 
	passDefaults = function(src, dst, props) {
		if (typeof props !== undefined && src && dst) {
			props = props.split(',');
			
			for (var i = 0; i < props.length; i++) {
				if (src.hasOwnProperty(props[i])) {
					dst[props[i]] = src[props[i]];
				}
			}
		}
	},
	
	/*
	 *	History management
	 */
 
	 addParam = function(params, title) {
	
		if (Modernizr.history) {
			var oldState = window.location.hash,
				state = oldState;
			
			if (state) {
				if (state.charAt(0) === '#') {
					state = state.substring(1);
				}
				state = state.objectify();
				state = extend(state, params);
			} else {
				state = params;
			}
			
			state = '#' + paramize(state);
			
			if (state !== oldState) {
				history.pushState(null, (typeof title === 'undefined')? null : title, isEmpty(state)? 'index.html' : state);
			}
		}
	},
	
	setParam = function(params, title) {
	
		if (Modernizr.history) {
			var state = '#' + paramize(params);
			
			if (state !== window.location.hash) {
				history.pushState(null, (typeof title === 'undefined')? null : title, isEmpty(state)? 'index.html' : state);
			}
		}
	},

	
	removeParam = function(param, title) {
		
		if (Modernizr.history) {
			var state = window.location.hash;
			
			if (state) {
				if (typeof param === 'undefined') {
					state = null;
				} else {
					if (state.charAt(0) === '#') {
						state = state.substring(1);
					}
					state = state.objectify();
					if (state.hasOwnProperty(param)) {
						delete state[param];
					}
				}
				history.pushState(null, (typeof title === 'undefined')? null : title, isEmpty(state)? 'index.html' : ('#' + paramize(state)));
			}
		}
	},
	
	removeSearch = function(title) {
		
		if (Modernizr.history) {
			history.replaceState(null, title || null, window.location.href.replace(window.location.search, ''));
		}
	},
	
	readParam = function() {
		
		return window.location.hash.objectify();
		
	},
	
	// Printing an image with caption
	
	printImage = function(src, title, caption) {
		
		if (!src) {
			return;
		}
		
		var pw = window.open('about:blank', 'print', 'location=no,status=no,titlebar=no');
		
		pw.document.open();
		pw.document.write('<!DOCTYPE html>\n<html>\n<head>\n<meta charset="utf-8">\n<title>' + (title || 'Print') + '</title>' +
			'\n<script>printOut=function(){window.print();window.close();}</scr' + 'ipt>' + 
			'\n<style>body{margin:0;padding:0;text-align:center;overflow:hidden;}\nimg{display:block;width:100%;height:auto;vertical-align:top;}</style>' +
			'\n</head>\n<body onLoad="setTimeout(printOut,100)">' +
			'<img src="' + src + '">' + (caption || '') + 
			'</body>\n</html>');
		pw.document.close();
	};


		
var log = function() {},
	DEBUG = (typeof DEBUG === 'undefined')? false : DEBUG;

// Waiting for jQuery loaded

(function($, undefined) {
		
	/*	
	 *	Debugging functions
	 */
	
	// log: logging function
	
	var _logel, _logover = false, _lastlog, _lastcnt = 1;
	
	log = function(c) {
		
		var resolveObject = function(c) {
			if ($.isArray(c)) {
				var s = '';
				for (var i = 0; i < c.length; i++) {
					s += resolveObject(c[i]) + ', ';
				}
				return '[ ' + s.substring(0, s.length-2) + ' ]';
			} else if (typeof c === 'object') {
				var s = '';
				for (var i in c) {
					s += i + ': ' + resolveObject(c[i]) + ',<br>';
				}
				return '{ ' + s + ' }';
			} else if (isNaN(c)) {
				return c;
			} else {
				return (parseInt(c) === c)? c : c.toFixed(4);
			}
		};
					
		if ( !DEBUG || _logover ) {
			return;
		}
		if ( !_logel ) {
			_logel = $('<div id="log" style="position:fixed;left:0;top:0;width:200px;bottom:0;overflow:auto;padding:10px;background-color:rgba(0,0,0,0.5);color:#fff;font-size:0.75em;z-index:99999"></div>').hover(function(){
				_logover = true;
			}, function(){
				_logover = false;
			}).appendTo('body');
		}
		if (c === _lastlog) {
			_logel.children(':first').empty().html(_lastlog + ' <sup>(' + (++_lastcnt) + ')</sup>');
		} else {
			$('<div style="height:3em;overflow:auto;">' + resolveObject(c) + '</div>').prependTo(_logel);
			_lastlog = c;
			_lastcnt = 1;
		}
	};
	
	// logEvents :: debugging events
	
	$.fn.logEvents = function( e ) {
		if ( !DEBUG ) {
			return;
		}
		
		var events = e || 'mousedown mouseup mouseover mouseout mousewheel dragstart click blur focus load unload reset submit change abort cut copy paste selection drag drop orientationchange touchstart touchmove touchend touchcancel MSPointerDown MSPointerMove MSPointerUp gesturestart gesturechange gestureend';

		return this.each(function() {
			$(this).on(events, function(e) {
				if (typeof e === UNDEF) {
					log('Undefined event');
				} else if (e.target) {
					if (e.target.id !== 'log') { 
						log(e.type + ' <span style="padding:0 4px;font-size:0.75em;background-color:#000;border-radius:4px;"><b>' + (e.target.nodeName? e.target.nodeName.toLowerCase() : '???') + '</b>' + (e.target.id? (':'+e.target.id) : '') + '</span>' + 
							(e.relatedTarget? (' <span style="padding:0 4px;font-size:0.6em;background-color:#800;border-radius:4px;"><b>' + e.relatedTarget.nodeName.toLowerCase() + '</b>' + (e.relatedTarget.id? (':'+e.relatedTarget.id) : '') + '</span>') : ''));
					}
				} else {
					log('No event target!');
				}
				return true;
			});
		});
	};
	
	// logCss :: tracks css values until the element is live
	
	$.fn.logCss = function( p, dur, step ) {
		if ( !DEBUG ) {
			return;
		}
		
		step = step || 20;
		dur = dur || 2000;
		var t0 = new Date();
		
		return this.each(function() {
			var el = $(this);
			var show = function( nm ) {
				var t = new Date() - t0;
				log(t + '&nbsp;::&nbsp;' + nm + ' = ' + el.css(nm));
				if (t > dur) {
					clearInterval(iv);
				}
			};
			var iv = setInterval(function() {
				if ( $.isArray(p) ) {
					for (var i = 0; i < p.length; i++) {
						show(p[i]);
					}
				}
				else {
					show(p);
				}
			}, step);
		});
	};
	
	// Making $.when working with arrays
	
	if ($.when.all === undefined) {
		$.when.all = function(deferreds) {
			var deferred = new $.Deferred();
			$.when.apply($, deferreds).then(
				function() {
					deferred.resolve(Array.prototype.slice.call(arguments));
				},
				function() {
					deferred.fail(Array.prototype.slice.call(arguments));
				}
			);
			return deferred;
		}
	}
	
	
	$.fn.waitAllImg = function(doneFn, successFn, failFn) {
		var self = $(this),
			deferreds = [],
			loadImage = function(image) {
				var deferred = new $.Deferred(),
					img = new Image();
				img.onload = function() {
					deferred.resolve(image);
				};
				img.onerror = function() {
					deferred.reject(new Error('Image not found: ' + image.src));
				};
				img.src = image.src;
				return deferred;
			},
			loadImages = function(imgs) {
				imgs.filter('img[src][src!=""]').each(function() {
					deferreds.push(loadImage(this));
				});
				return $.when.all(deferreds);
			};

		loadImages(self).then(
			function(self) {
				if ($.isFunction(successFn)) {
					successFn.call(self);
				}
			},
			function(err) {
				if ($.isFunction(failFn)) {
					failFn.call(err);
				}
			}
		).then(function() {
			if ($.isFunction(doneFn)) {
				doneFn.call(self);
			}
		});
		return this;
	};
	
	// Wait for all images loaded
	/*
	$.fn.waitAllImg = function(doneFn) {
		var deferreds = [];
		
		$.when.apply($, deferreds).done(
			function() {
				$(this).off('.waitAll');
				if ($.isFunction(doneFn)) {
					doneFn.call(this);
				}
			}	
		);
		
		return this.each(function() {
			var img = $('<img>'),
				deferred = new $.Deferred();
			deferreds.push(deferred);
			img.on('load.waitAll', function() {
				deferred.resolve();
			}).attr('src', $(this).attr('src'));
		});
	}
	*/
	
	/*
	 *	Google Maps functions
	 *	canvas is a jQuery element with data('location') and other optional parameters
	 */

	loadMap = function(canvas) {
		
		if ( typeof google === 'undefined' || !google.maps ) {
			// No API is loaded yet
			var script = document.createElement('script');
			
			script.type = 'text/javascript';
			script.src = 'https://maps.googleapis.com/maps/api/js?' + ((typeof googleApiKey === 'undefined')? '' : ('key=' + googelApiKey)) + '&callback=showMap';
			canvas.attr('id', 'map-canvas');
			document.body.appendChild(script);
			return;
		}
		
		showMap(canvas);
	},
		
	showMap = function(canvas) {
		canvas = canvas || $('#map-canvas');
		
		if (canvas) {
			var location = canvas.data('location');
			if (location) {
				location = location.split(',');
				var pos = new google.maps.LatLng(parseFloat(location[0]), parseFloat(location[1])),
					map = new google.maps.Map(canvas[0], {
						center: pos, 
						zoom: canvas.data('map-zoom') || 12,
						mapTypeId: canvas.data('map-type') || google.maps.MapTypeId.ROADMAP,
						panControl: false
					}),
					marker = new google.maps.Marker({
						position: pos, 
						map: map
					});
			}
		}
	};

	
})(jQuery);
