/*	
 *	addSwipe() :: Swipe gesture support
 *
 *	Copyright by Lazaworx
 *	http://www.lazaworx.com
 *	Author: Laszlo Molnar
 *
 *	Dual licensed under the MIT and GPL licenses.
 *	- http://www.opensource.org/licenses/mit-license.php
 *	- http://www.gnu.org/copyleft/gpl.html
 *
 *	Usage: $(element).addSwipe( leftFn, rightFn, options );
 *	Options: treshold, oversizeTreshold, margin
		treshold: 40,			// Considering as click instead of move
		oversizeTreshold: 0.15,	// The proportion of screen size moving within this boundary still don't trigger prev/next action 
		margin: 15				// Re-align to this margin, when moved over
 */
 
 (function($) {
	
	// Easing function by George Smith
	$.extend( jQuery.easing, {
		easeOutCubic: function (x,t,b,c,d) {
			return c*((t=t/d-1)*t*t+1)+b;
		}
	});
	
	$.fn.addSwipe = function( leftFn, rightFn, settings ) {
		
		settings = $.extend( {}, $.fn.addSwipe.defaults, settings );
														
		return this.each(function() {
			
			var t = $(this),
				tp = t.parent(),
				self = this;
				
			var x0 = 0, y0 = 0,			// storing transform offset
				cw = tp.outerWidth(),	// container width, height
				ch = tp.outerHeight(),	
				ew = t.outerWidth(), 	// element width, height
				eh = t.outerHeight(),
				mx,						// max x for panoramic
				uto,					// window resize update timeout
				mto,					// auto move timeout
				tt,						// touch start time
				ex0, ey0,				// event start coords
				dx, dy,					// relative move
				cax;					// constrain to x axis
			
			var updateClipDim = function() {
				cw = tp.outerWidth();
				ch = tp.outerHeight();
			}
			
			$(window).on('resize.addswipe', function() {
				clearTimeout(uto);
				uto = setTimeout(updateClipDim, 50);
			});
			
			var clicked = function(e) { //log('clicked '+(self.dragOn?'1':'0'));
				if (self.dragOn) {
					return false;
				} else {
					e.target.trigger('click', e);
				}
			}
			
			var move = function(e) {
				if (cax) {
					dx = ((e.touches && e.touches.length > 0)? e.touches[0].clientX : e.clientX) - ex0;
					t.translate(x0 + dx, 0);
				} else {
					if (e.touches && e.touches.length > 0) {
						dx = e.touches[0].clientX - ex0;
						dy = e.touches[0].clientY - ey0;
					} else if (e.clientX !== null ) {
						dx = e.clientX - ex0;
						dy = e.clientY - ey0;
					}
					t.translate(x0 + dx, y0 + dy);
				}
			};
						
			var touchEnd = function(e) { //log('touchEnd '+(self.dragOn?'1':'0'));
				swiped();
				self.removeEventListener(TOUCH_MOVE, touchMove);
				self.removeEventListener(TOUCH_END, touchEnd);
				return false;
			};
			
			var dragEnd = function(e) {
				e.preventDefault();
				swiped();
				$(document).off('mousemove.addswipe mouseup.addswipe');
				return false;
			};
			
			var touchMove = function(e) { //log('touchMove');
				e.preventDefault();
				move(e);
				return false;
			};
			
			var dragMove = function(e) {
				e.preventDefault();
				move(e.originalEvent);
				return false;
			};
			
			var touchStart = function(e) { //log('touchStart '+(self.dragOn?'1':'0'));
				if (!e.touches) {
					ex0 = e.clientX;
					ey0 = e.clientY;
				} else if (e.touches.length > 1) {
					// multi finger flick
					return true;
				} else {
					ex0 = e.touches[0].clientX;
					ey0 = e.touches[0].clientY;
				}
				startSwipe();
				self.addEventListener(TOUCH_MOVE, touchMove);
				self.addEventListener(TOUCH_END, touchEnd);
				return true;
			};
			
			var mouseDown = function(e) { //log('mouseDown '+(self.dragOn?'1':'0'));
				ex0 = e.originalEvent.clientX;
				ey0 = e.originalEvent.clientY;
			};
			
			var dragStart = function(e) { //log('dragStart '+(self.dragOn?'1':'0'));
				e.preventDefault();
				mouseDown(e);
				startSwipe();
				t.on('click.addswipe', clicked);
				$(document).on({
					'mousemove.addswipe': dragMove,
					'mouseup.addswipe': dragEnd
				});
				return false;
			};
			
			var startSwipe = function() {
				t.removeClass('smooth');
				self.dragOn = true;
				tt = new Date().getTime();
				cw = cw || tp.outerWidth();
				ew = t.outerWidth();
				eh = t.outerHeight();
				cax = eh <= ch;
				x0 = t.data('tr_x') || 0;
				y0 = cax? 0 : (t.data('tr_y') || 0);
				dx = dy = 0;
				//log('x0 = ' + x0);
				mx = (ew > cw)? Math.round((ew - cw * (1 - settings.oversizeTreshold)) / 2) : 0;
			};
						
			var swiped = function() { //log('swiped '+(self.dragOn?'1':'0'));
				//log('x0='+x0+' dx='+dx);
				
				if (cax && !mx && (Math.abs(dx) < settings.treshold)) {
					
					// Small move -> click
					self.dragOn = null;
					t.translate(0, 0);
					t.trigger('click');
					
				} else {
					
					var dt = (new Date().getTime() - tt),
						tx = x0 + dx + (dx * 300 / dt),
						ty = cax? 0 :  (y0 + dy + (dy * 300 / dt));
					//speed = 1000 * (cax? dx : Math.sqrt(dx * dx + dy * dy)) / ;
					//mto = setInterval(moveExtra, 50);
					t.translate(tx, ty, 'smooth');
					
					//move(Math.round(333 * dx / ts), cax? 0 : Math.round(333 * dy / ts), 500);
					//log('swiped:'+Math.round(tx)+' / '+mx);
					if ( tx < -mx ) {
						if ( $.isFunction(leftFn) ) {
							//log('leftFn');
							leftFn.call(self); 
						}
					} else if ( tx > mx ) {
						if ( $.isFunction(rightFn) ) {
							//log('rightFn');
							rightFn.call(self);
						}
					} else {
					}
					
				}
				
				setTimeout(function() { 
					self.dragOn = null;
				}, 100);
				
				return false;
			};
			
			t.attr('draggable', 'true');
			
			if (document.touchMode) {
				self.addEventListener(TOUCH_START, touchStart);
			} else {
				t.on({
					'dragstart.addswipe': dragStart,
					'mousedown.addswipe': mouseDown
				});
			}
			
			var dragCancel = function() { //log('dragCancel '+(self.dragOn?'1':'0'));
				setTimeout(function() { 
					self.dragOn = null;
				}, 20);
				t.translate(0, 0, 'smooth');
				return false;
			};
			
			t.on('dragcancel.addswipe', dragCancel);
			
			var unSwipe = function() { //log('unSwipe '+(self.dragOn?'1':'0'));
				setTimeout(function() { 
					self.dragOn = null;
				}, 20);
				t.removeAttr('draggable');
				if ( document.touchMode ) {
					self.removeEventListener(TOUCH_MOVE, touchMove);
					self.removeEventListener(TOUCH_END, touchEnd);
					self.removeEventListener(TOUCH_START, touchStart);
				} else {
					t.off('.addswipe');
					$(document).off('.addswipe');
				}
			};
			
			// Unswipe - removing the swipe event
			t.on('unswipe.addswipe', unSwipe);
			
			// No select
			t.on('selectstart.addswipe', noAction); 

		});
	};
	
	$.fn.addSwipe.defaults = {
		treshold: 40,
		oversizeTreshold: 0.25,
		margin: 15
	};

})(jQuery);
